# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Tests for utility functions."""

from datetime import datetime

from invenio_app_rdm.records_ui.utils import set_default_value


def test_set_default_value__value():
    """Test setting a value."""
    dict_ = {"metadata": {"publication_date": None}}
    value = "1939-01-01"
    path = "publication_date"

    set_default_value(dict_, value, path)

    assert dict_["metadata"]["publication_date"] == value


def test_set_default_value__callable():
    """Test setting a value via a callable."""
    dict_ = {"metadata": {"publication_date": None}}

    def func():
        return datetime.now().strftime("%Y-%m-%d")

    path = "publication_date"
    today = func()

    set_default_value(dict_, func, path)

    assert dict_["metadata"]["publication_date"] == today


def test_set_default_value__no_path_prefix():
    """Test setting a value for an unprefixed path."""
    dict_ = {"metadata": {"publication_date": None}}
    value = "1939-01-01"
    path = ".publication_date"

    set_default_value(dict_, value, path)

    assert dict_["metadata"]["publication_date"] is None
    assert dict_["publication_date"] == value


def test_set_default_value__explicit_and_automatic_prefix():
    """Compare values set with auto-prefix and explicit prefix."""
    dict1 = {"metadata": {"publication_date": None}}
    dict2 = dict1.copy()
    value = "1939-01-01"
    path1 = "publication_date"
    path2 = ".metadata.publication_date"

    set_default_value(dict1, value, path1)
    set_default_value(dict2, value, path2)

    assert (
        dict1["metadata"]["publication_date"]
        == dict2["metadata"]["publication_date"]
    )
    assert dict1["metadata"]["publication_date"] == value



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio App RDM."""

__version__ = '9.0.0.dev12'

__all__ = ('__version__', )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Request views module."""

from flask import current_app, g, render_template
from flask_babelex import lazy_gettext as _
from flask_login import login_required
from invenio_communities.members.services.request import CommunityInvitation
from invenio_communities.proxies import current_communities
from invenio_communities.views.decorators import pass_community
from invenio_rdm_records.proxies import current_rdm_records_service
from invenio_rdm_records.requests import CommunitySubmission
from invenio_rdm_records.resources.serializers import UIJSONSerializer
from invenio_records_resources.services.errors import PermissionDeniedError
from invenio_requests.views.decorators import pass_request
from jinja2 import TemplateNotFound
from sqlalchemy.orm.exc import NoResultFound


def requests_detail(request=None, default_query_config=None):
    """Requests detail page."""
    # temporarily, until serializers and avatars implemented
    try:
        user_email = request["created_by"]["user"]
        request["created_by"].update({
            "avatar": "/static/images/placeholder.png",
            "full_name": user_email
        })
    except KeyError:
        request["created_by"].update({
            "avatar": "/static/images/placeholder.png",
            "full_name": request["created_by"]["community"]
        })

    topic_record = None
    is_draft = None
    permissions = []

    is_topic_record_type = 'record' in request["topic"]

    if is_topic_record_type:
        try:
            topic_record = current_rdm_records_service.read_draft(
                id_=request["topic"]["record"], identity=g.identity)

        except NoResultFound:
            topic_record = current_rdm_records_service.read(
                id_=request["topic"]["record"], identity=g.identity)

        permissions = topic_record.has_permissions_to(
            ['edit', 'new_version', 'manage',
             'update_draft', 'read_files'])
        topic_record = UIJSONSerializer().serialize_object_to_dict(
            topic_record.data)
        is_draft = topic_record["is_draft"]
        request["topic"] = topic_record

        # end temporary block

        return render_template(
            f"invenio_requests/community-submission/index.html",
            # TODO: implement and use request UI serializer
            request=request,
            record=topic_record,
            is_preview=True,
            is_draft=is_draft,
            permissions=permissions,
            files=[],
            default_query_config=default_query_config
        )


def invitation_details(community_pid_value=None,
                       request=None, default_query_config=None):
    """Invitation details page."""
    community = current_communities.service.read(
        id_=community_pid_value, identity=g.identity
    )

    permissions = community.has_permissions_to(
        ['update', 'read', 'search_requests', 'search_invites']
    )
    if not permissions['can_search_invites']:
        raise PermissionDeniedError()

    return render_template(
        f"invenio_requests/community-invitation/index.html",
        community=community.to_dict(),
        types={
            "organization": _("Organization"),
            "event": _("Event"),
            "topic": _("Topic"),
            "project": _("Project")
        },
        permissions=permissions,
        request=request,
        default_query_config=default_query_config,
    )


@login_required
@pass_request
def requests_view(request=None, **kwargs):
    """General view for requests."""
    request_dict = request.to_dict()
    request_type = request_dict.get('type')
    community_pid_value = kwargs.get('community_pid_value')
    default_query_config = dict(
        size=current_app.config['REQUESTS_TIMELINE_PAGE_SIZE']
    )

    if request_type == CommunitySubmission.type_id:
        return requests_detail(request=request_dict,
                               default_query_config=default_query_config)

    elif request_type == CommunityInvitation.type_id and community_pid_value:
        return invitation_details(community_pid_value=community_pid_value,
                                  request=request_dict,
                                  default_query_config=default_query_config)

    else:
        return render_template("invenio_requests/details/index.html",
                               request=request_dict,
                               default_query_config=default_query_config)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Request UI blueprints module."""

from flask import Blueprint
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError
from invenio_records_resources.services.errors import PermissionDeniedError
from invenio_requests.views.ui import not_found_error, \
    record_permission_denied_error, record_tombstone_error

from invenio_app_rdm.requests_ui.views.requests import invitation_details, \
    requests_detail, requests_view


def create_ui_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config["RDM_REQUESTS_ROUTES"]

    blueprint = Blueprint(
        "invenio_app_rdm_requests",
        __name__,
        template_folder="../templates",
        static_folder='../static'
    )

    blueprint.add_url_rule(
        routes["user-dashboard-community-submission"],
        view_func=requests_view,
    )

    blueprint.add_url_rule(
        routes['invitation_details'],
        view_func=requests_view
    )

    blueprint.add_url_rule(
        routes['request_details'],
        view_func=requests_view
    )

    # Register error handlers
    blueprint.register_error_handler(
        PermissionDeniedError, record_permission_denied_error)
    blueprint.register_error_handler(PIDDeletedError,
                                     record_tombstone_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)

    return blueprint



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 Northwestern University.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test deposit views."""

import pytest
from elasticsearch_dsl import Q
from invenio_access.permissions import system_identity
from invenio_vocabularies.proxies import current_service as vocabulary_service
from invenio_vocabularies.records.api import Vocabulary

from invenio_app_rdm.records_ui.views.deposits import VocabulariesOptions


@pytest.fixture()
def additional_resource_types(running_app):
    """Resource type vocabulary record."""
    vocabulary_service.create(system_identity, {
        "id": "publication",
        "icon": "file alternate",
        "props": {
            "csl": "report",
            "datacite_general": "Text",
            "datacite_type": "",
            "eurepo": "info:eu-repo/semantics/other",
            "openaire_resourceType": "17",
            "openaire_type": "publication",
            "schema.org": "https://schema.org/CreativeWork",
            "subtype": "",
            "type": "publication",
        },
        "title": {
            "en": "Publication"
        },
        "tags": ["depositable", "linkable"],
        "type": "resourcetypes"
    })
    vocabulary_service.create(system_identity, {
        "id": "publication-annotationcollection",
        "icon": "file alternate",
        "props": {
            "csl": "report",
            "datacite_general": "Collection",
            "datacite_type": "",
            "eurepo": "info:eu-repo/semantics/technicalDocumentation",
            "openaire_resourceType": "9",
            "openaire_type": "publication",
            "schema.org": "https://schema.org/Collection",
            "subtype": "publication-annotationcollection",
            "type": "publication",
        },
        "title": {
            "en": "Annotation collection"
        },
        "tags": ["depositable", "linkable"],
        "type": "resourcetypes"
    })

    Vocabulary.index.refresh()


def test_resource_types(additional_resource_types):
    options = VocabulariesOptions()
    expected = [
        # If no corresponding parent type entry (e.g. image-photo), type_name
        # is own title and subtype_name is empty.
        {
            "icon": "chart bar outline",
            "id": "image-photo",
            "subtype_name": "",
            "type_name": "Photo",
        },
        # If parent type is itself (e.g. Publication), type_name is own title
        # and subtype_name is empty.
        {
            "icon": "file alternate",
            "id": "publication",
            "subtype_name": "",
            "type_name": "Publication",
        },
        # If corresponding parent type entry, type_name is parent's title
        # and subtype_name is own title.
        {
            "icon": "file alternate",
            "id": "publication-annotationcollection",
            "subtype_name": "Annotation collection",
            "type_name": "Publication",
        }
    ]

    # the choice of this filter isn't important for the test
    result = options._resource_types(Q('term', tags="depositable"))

    sorted_result = sorted(result, key=lambda e: e["id"])
    assert expected == sorted_result


def test_dump_subjects_vocabulary(running_app):
    options = VocabulariesOptions()
    expected = {
        "limit_to": [
            {"text": "All", "value": "all"},
            {"text": "MeSH", "value": "MeSH"},
        ]
    }

    result = options.subjects()

    assert expected == result


def test_dashboard_routes(app, client_with_login):
    """Test dashboard routes to be only the configured ones."""
    client = client_with_login
    response = client.get("/me")
    assert response.status_code == 200

    for r in app.config["_DASHBOARD_ROUTES"]:
        response = client.get("/me/{0}".format(r))
        assert response.status_code == 200

    response = client.get("/me/not-existing")
    assert response.status_code == 404
